;;;
;;; hardware.asm
;;;	Initialize the hardware for the game.
;;;
#include "portdef.h"
#include "regdef.h"
#include "utility.mac"

;;;
;;; a little short hand to make the code more readable
;;; 
#define AsOutput	bcf
#define AsInput		bsf


	global	HardwareInit	; sets up all hardware
	
PROG1	CODE

;;;*******************************************************************
;;; NAME:	HardwareInit()
;;;
;;; DESCR:	Initialize the I/O ports and other such things.
;;;
;;; ARGS:
;;;
;;; RETURNS:
;;;
;;; NOTES:
;;;*******************************************************************
HardwareInit:

	BankSelect	0

	;; need to get Port A in shape for standard I/O (trying NOT
	;;	to break the abstraction of where things are)

	clrf		PORTA		; clears the latches
	movlw		0x07
	movwf		CMCON		; with previous, clears comparator mode
	
	;; set the data in output latches to a known state
	;; only the column data is important at this point

	bcf		DISPLAY_0
	bcf		DISPLAY_1
	bcf		DISPLAY_2
	bcf		DISPLAY_3
	bcf		DISPLAY_4

	;; and we'll have a blank display

	bsf		DISPLAY_LATCH

	;; the sound line is just ina low state

	bcf		SOUND_PORT
	
	BankSelect	1
	
	;; initialize the display port
	
	AsOutput	DISPLAY_0_CTL
	AsOutput	DISPLAY_1_CTL
	AsOutput	DISPLAY_2_CTL
	AsOutput	DISPLAY_3_CTL
	AsOutput	DISPLAY_4_CTL
	AsOutput	DISPLAY_5_CTL
	AsOutput	DISPLAY_6_CTL

	;; initialize display latch

	AsOutput	DISPLAY_LATCH_CTL

	;; buttons

	AsInput		BUTTON_0_CTL
	AsInput		BUTTON_1_CTL

	;; sound

	AsOutput	SOUND_PORT_CTL

	BankSelect	0

	bcf		INTCON,GIE	; prevent interrupts

	clrwdt				; go ahead and clear it
	
	return

;;; =======================================================================
;;; =======================================================================

	END
