;;
;; random.asm
;;
;;   Implements some sort of randomness.
;;
;; This implements the simple randomness associated with user's button
;; presses.  Timer 0 is used as the random register, and it is reset
;; periodically.  It runs at the speed of the instruction clock, which
;; is quite zippy, so the delays between button press/release will
;; (hopefully) be a good random number.
;;
;; Check random.mac for a few macros that make using random numbers easy.
;;

#include "portdef.h"
#include "utility.mac"

	global	RandomInit	; initialize random generator (TIMER0)

PROG1	CODE

;;
;; RandomInit() - initializes the "random" number generator.
;;
;;		The Random Number Generator is really just TIMER0 set
;;		to run as fast as it can.  Randomness comes from the
;;		unpredictable user press of the buttons.
;;
RandomInit:
		bcf	INTCON,T0IE	; clear timer interrupting (though GIE should be off anyway)

		BankSelect	1

		bcf	OPTION_REG,T0CS	; timer on internal clock
		bsf	OPTION_REG,PSA	; by assigning the pre-scale to WDT, we get 1:1 on TIMER0
					;  pre-scaler settings then don't matter

		BankSelect	0

		return

;;; =======================================================================
;;; =======================================================================

	END
