;;
;; Register definintions
;;

;;
;; WORK registers
;;
#define	WORK0	0x20
#define WORK1	0x21
#define WORK2	0x22
#define WORK3	0x23
#define WORK4	0x24

#define ARG0	0x25
#define ARG1	0x26

#define SAVEW	0x27

;;
;; Main loop delay factor and register
;;
#define MAIN_DELAY_DEFAULT	0xff
#define MAIN_COUNTER		0x28

;;
;; Score registers
;;
#define	SCORE_HIGH	0x29
#define	SCORE_LOW	0x2a
#define SCORE_BCD	0x2b
#define SCORE_BCD_0	(SCORE_BCD+0)
#define SCORE_BCD_1	(SCORE_BCD+1)
#define SCORE_BCD_2	(SCORE_BCD+2)
#define SCORE_BCD_3	(SCORE_BCD+3)
#define SCORE_BCD_4	(SCORE_BCD+4)

;;
;; CURCOL - the current column being displayed.  This is used to index
;;	    into the COLDATA "array" using the index register.
;;	    This is used exclusively to keep track of what is being
;;	    displayed and SHOULD NOT be used anywhere else.
;;
CURCOL	EQU		0x30

;;
;; COLDATA - column data is kept in an "array" which is indexed by the
;;	     index register.  It now includes space for 2 bits of blink
;;	     data in between the columns.  No definition of these two
;;	     bits as of yet.
;;
#define COLDATA_BASE	0x31
#define COLDATA_0	COLDATA_BASE+(0*3)
#define COLDATA_1	COLDATA_BASE+(1*3)
#define COLDATA_2	COLDATA_BASE+(2*3)
#define COLDATA_3	COLDATA_BASE+(3*3)
#define COLDATA_4	COLDATA_BASE+(4*3)

;;
;; Button Registers
;;
#define BUTTON_STATE_0		0x40
#define BUTTON_LAST_STATE_0	0x41
#define BUTTON_STATE_COUNTER_0	0x42

#define BUTTON_STATE_1		0x43
#define BUTTON_LAST_STATE_1	0x44
#define BUTTON_STATE_COUNTER_1	0x45

;;
;; Scrolling Text Registers
;;	TEXT_SOURCE - zero is table text, non-zero is score display
;;	TEXT_TYPE - scrolling or flashing
;;	TEXT_LOC - contains the location of the text string
;;	TEXT_INDEX - index to the current char within the above text string
;;	TEXT_STATE - where we are in the scrolling of the text on the display
;;	TEXT_SPEED - the desired scrolling speed
;;	TEXT_COUNTER - the current count on the way to the given speed
;;	TEXT_COUNTER2 - a free-running counter that gives us a 2X on TEXT_COUNTER
;;	TEXT_CHAR[12] - the current char (1) and next char loaded for use later
;;	TEXT_CHAR_SHORT - used to remember if a char is 4 columns instead of 5
;;
#define TEXT_DEFAULT_SPEED	d'125'
#define	TEXT_TYPE	0x46
#define	TEXT_LOC_HIGH	0x47
#define	TEXT_LOC_LOW	0x48
#define	TEXT_INDEX	0x49
#define	TEXT_STATE	0x4a
#define TEXT_SPEED	0x4b
#define TEXT_COUNTER	0x4c
#define TEXT_COUNTER2	0x4d
#define	TEXT_CHAR1	0x4e
#define	TEXT_CHAR2	0x4f
#define	TEXT_CHAR_SHORT	0x50
#define TEXT_SOURCE	0x51

;;
;; Blink setting - the current setting for bits that are being
;;			written to the display.  Can be 0 through 3.
;;
#define	BLINK_SETTING	0x52

;;
;; Blink data - there is a base blink rate which is a define, which is used
;;		as a multiple of the column refresh rate, which increments
;;		the BLINK_COUNTER.  When the BLINK_COUNTER hits zero, it
;;		increments the BLINKER register, which is then used to
;;		differentiate the different speed of blinks.
;;
#define BLINK_RATE	d'20'
#define BLINK_COUNTER	0x53
#define	BLINKER		0x54

;;
;; Sound Registers
;;
#define SOUND_ON	0x55
#define SOUND_FREQ	0x56
#define SOUND_STATE	0x57
#define SOUND_DURATION	0x58
#define MUSIC_INNER	0x59
#define MUSIC_OUTER	0x5a
#define MUSIC_LOC_HIGH	0x5b
#define MUSIC_LOC_LOW	0x5c
#define MUSIC_LENGTH	0x5d
#define MUSIC_STACCATO	0x5e

;;
;; Undefined for now - reserved for the engine
;;
#define UNDEF3		0x5f

;;
;; 0x60 through 0x7f are reserved for application games
;;
