;; retris.asm!
;;

#include "regdef.h"
#include "portdef.h"
#include "display.h"
#include "display.mac"
#include "button.h"
#include "button.mac"
#include "sound.h"
#include "text.h"
#include "text.mac"
#include "music.mac"
#include "utility.mac"
#include "music.h"
#include "musicdata.h"
#include "score.h"
#include "score.mac"
#include "random.h"
#include "retrisregs.h"

;; FILE-WIDE DEFINES
;;
;;	DOTS - the number of dots in each piece
;;
#define	DOTS	3

	global	GameInit
	global	GameProcess

;;	global	RetrisInit
;;	glocal	RetrisProcess

PROG1	CODE


#define NODATA	0x88


;;;
;;; DotProcess() - MACRO - this macro is used to iteratively process the dots of a
;;;			piece.  It is called with a register that has the piece
;;;			as well as a register that has the current position of the
;;;			piece.  
;;;
;;;			This routine will check the (X,Y) of the dot to see if it
;;;			lies between 0 and the two X and Y targets.  Normally the
;;;			X is simply MAXX, but Y can be either MAXY when processing
;;;			for display, or MAXY_EXTRA which includes rows above the
;;;			display screen which will be occupied by pieces as they fly
;;;			in from the top of the screen.
;;;
;;;			This macro is used in conjunction with DotProcessNext() which
;;;			should be called after processing a dot to process the next
;;;			dot.  Note that this whole mechanism counts on there being at
;;;			least one valid dot in each piece (a good assumption).
;;;
;;;			This macro causes Z to be set (FALSE) if the dot in W lies
;;;			outside of the given bounds.  Z will be clear (TRUE) if the
;;;			dot in W is within bounds.  Where W is a dot (x,y) within
;;;			the target piece.
;;;
;;;			ARG0 is used, as is WORK0
;;;
;;;			Note that the given piece is not change, note too that it
;;;			CAN be given as ARG0
;;;
#define DOTLOOP		RETRIS_DOTLOOP
#define DotProcessLoop	DP#v(1)id
#define DotProcessOOB	DP#v(2)id
#define DotProcessEnd	DP#v(3)id
#define DotProcessNext2	DP#v(4)id
			
DotProcess:	macro	id,piece,position,maxx,maxy

		;;       id - a unique id used for labels
		;;    piece - a REGISTER with the piece index
		;; position - a REGISTER with the piece position
		;;     maxx - a constant max x
		;;     maxy - a constant max y

		movf	piece,W
		movwf	ARG0

		movlw	DOTS
		movwf	DOTLOOP
DotProcessLoop:
		decf	DOTLOOP,W	; set up to get a dot, ARG0 has piece number
		call	PieceGetDot	; get the W dot from ARG0 piece, W will have the coords upon return

		xorlw	NODATA		; check W for a valid point
		skpnz
		goto	DotProcessNext2	; a "blank" dot in the piece

		xorlw	NODATA		; get the returned coordinates back

		;; check the indicated dot to see if it lies within the provided bounds
		;; 0 <= X <= MAXX, 0 <= Y <= MAXY

		addwf	position,W	; W will now have the coords to check
		movwf	SAVEW

		;; check Y

		andlw	0x0f		; mask off X bits
		sublw	maxy		; subtract target Y from MAXY
		btfss	STATUS,C
		goto	DotProcessOOB	; a dot of the piece lies out of bounds

		;; check X

		swapf	SAVEW,W		; swap the X bits 
		andlw	0x0f		; and mask off Y bits
		sublw	maxx		; subtract target X from MAXX
		btfss	STATUS,C
		goto	DotProcessOOB	; a dot of the piece lies out of bounds

		movf	SAVEW,W
		bcf	STATUS,Z	; set up for TRUE (Z is clear)
		goto	DotProcessEnd

DotProcessOOB:
		movf	SAVEW,W
		bsf	STATUS,Z	; set up for FALSE (Z is set)

DotProcessEnd:
		;; at this point if Z ("FALSE") is set if the point was out of bounds
		;;   if Z is NOT set ("TRUE") then the point was in bounds
		;; - in either case W has the current dot location

		endm

;;
;; DotProcessNext() - MACRO - used to process the next dot.  This routine is a no-op
;;				if there are no more dots to process.

DotProcessNext:	macro	id
DotProcessNext2:
		decfsz	DOTLOOP
		goto	DotProcessLoop	
		endm

ScoreAddClear:	macro
		ScoreAddMac	d'200'
		ScoreAddMac	d'200'
		ScoreAddMac	d'100'
		endm

ScoreAddLevel:	macro
		ScoreAddMac	d'100'
		endm

ScoreAddDown:	macro
		ScoreAddMac	d'10'
		endm

ScorePieceMove:	macro
		ScoreSubMac	d'1'
		endm

GameInit:
RetrisInit:
	clrf	RETRIS_STATE
	clrf	RETRIS_SPEED
	return

RetrisGameInit:
	clrf	RETRIS_GSTATE
	clrf	RETRIS_POS
	clrf	RETRIS_PIECE
	clrf	RETRIS_DONE
	clrf	RETRIS_LEVEL
	clrf	RETRIS_BEEPER
	movlw	0		; always maintain a zero blink environment
	call	BlinkSetting
	call	DisplayClear
	call	ScoreInit
	return
		
;;;
;;; Top-level state dispatch tables
;;; 
	StateDispatchTable	Retris,RETRIS_STATE,d'8'
	StateDispatchTable	RetrisG,RETRIS_GSTATE,d'9'

;;;
;;; RetrisProcess() - the top-level processing routine called by the RetroGame
;;;			engine.  Maintains the overall state of the game.
;;; 
GameProcess:
RetrisProcess:
	StateDispatch	Retris,RETRIS_STATE

;;;
;;; RetrisGameProcess() - the game processing routine.  All things related to the
;;;			game occur here - as opposed to welcome text, display of
;;;			the score, etc.
;;; 
RetrisGameProcess:
	StateDispatch	RetrisG,RETRIS_GSTATE

;;; ========================================================================
;;; RetrisProcess (top-level) states
;;; ========================================================================

Retris0state:
	;;
	;; the "waiting to play" state...just starts up the welcome message.
	;; not much happens in this state except starting the message
	;; it jumps right to the next state
	;;
	TextScrollInit	Welcome
	StateNext	RETRIS_STATE
	return

Retris1state:
	;;
	;; In this state we wait for a button to be pressed, in which case
	;; we jump to starting the game.  Otherwise we just wait for the
	;; text to be done, and we restart it by going to state 0.
	;;
	ButtonSkipIfNotPressed	0
	goto		Retris1state2
	ButtonAck		0

	ButtonSkipIfNotPressed	1
	goto		Retris1state2
	ButtonAck		1

	TextDoneSkip
	return
	StateSet	RETRIS_STATE,0
	return
	
Retris1state2:
	;;
	;; now we're ready to start the game...but first, display countdown
	;; ...note that we stop the current
	;; text before moving to the next state

	TextStop
	TextFlashInit	CountDown
	StateNext	RETRIS_STATE
	return
	
Retris2state:
	;;
	;; Wait for the text to be done, play the "charge" music, reset the game
	;; settings, and jump into the game!

	TextDoneSkip
	return

	MusicScorePlay	Charge
	call		RetrisGameInit
	StateNext	RETRIS_STATE
	return
	
Retris3state:
	call		RetrisGameProcess

	;; there are no "winners" in this game...you just try
	;; to stave off losing for as long as possible
	;; The above sets RETRIS_DONE to non-zero when the game is done.

	movf	RETRIS_DONE
	skpnz			; if not done with the game, just return
	return

	;; otherwise, the game is over, so proceed to score display
	
	StateNext	RETRIS_STATE
	return
	
Retris4state:
	;; the game is over
	
	MusicScorePlay	Dirge
	StateNext	RETRIS_STATE
	return
	
Retris5state:
	TextScrollInit	ScoreLeadin
	StateNext	RETRIS_STATE
	return

Retris6state	
	;; wait for button press while displaying score leadin
	
	ButtonSkipIfNotPressed	0
	goto		Retris6state2
	ButtonAck		0

	ButtonSkipIfNotPressed	1
	goto		Retris6state2
	ButtonAck		1

	TextDoneSkip
	return
	call		TextScoreScrollInit
	StateNext	RETRIS_STATE
	return

Retris6state2:
	StateSet	RETRIS_STATE,1
	goto		Retris1state2
	
Retris7state:
	;; wait for button press while displaying score
	
	ButtonSkipIfNotPressed	0
	goto		Retris7state2
	ButtonAck		0

	ButtonSkipIfNotPressed	1
	goto		Retris7state2
	ButtonAck		1

	TextDoneSkip
	return
	StateSet	RETRIS_STATE,5
	return

Retris7state2:
	StateSet	RETRIS_STATE,1
	goto		Retris1state2

	
;;; ========================================================================
;;; RetrisGameProcess (low-level) states
;;; ========================================================================

RetrisG0state:
	call		PieceNew	; starts above the screen
	call		PieceMoveDown	; move it down one if possible
	skpnz
	goto		PieceHitBottom	; will definitely cause game over in this case

	call		PieceDisplay

	call		RetrisSpeed		; set the speed for the first incoming piece
	movwf		RETRIS_SPEED

	StateNext	RETRIS_GSTATE
	return

RetrisG1state:
	;; processing state, we stay here most of the time, moving pieces
	;; there is a delay here based upon the current speed (which changes)
	;; there is a check for filled lines here which launches into a new state

	call	RetrisButtons

	decfsz	RETRIS_SPEED
	return

	;; here is the standard "beeper" like in retrorover, one beep per move down

	incf	RETRIS_BEEPER	; this counter just wraps and wraps, beeping all the way
	movf	RETRIS_BEEPER,W
	andlw	0x01		; get the last bit for the sound pointer
	addlw	3
	call	SoundNote
	movlw	d'128'
	call	SoundDuration

	ScoreAddDown		; get some points just for a piece moving

	call	RetrisSpeed
	movwf	RETRIS_SPEED

	call	PieceClear
	call	PieceMoveDown
	skpnz
	goto	PieceHitBottom	; since this is a GOTO it does the return
	call	PieceDisplay

	return

RetrisG2state:
	call		FilledRowCheck		; check for a filled line, if Z set (FALSE) none exist
	skpnz
	goto		RetrisG2state2

	;; at this point, W has the index (Y) of the line that should be removed

	call		FilledRowFlash

	movlw	1				; quick high beep for flashing row
	call	SoundNote
	movlw	d'64'
	call	SoundDuration

	call		RetrisSpeed		; flashing lasts for the same amount of time as
	movwf		RETRIS_SPEED		;   does the rows as they go by
	
	StateNext	RETRIS_GSTATE
	return

RetrisG2state2:				;; indicates done with filled lines, go back for new piece
	StateSet	RETRIS_GSTATE,0
	return

RetrisG3state:

	decfsz		RETRIS_SPEED	; wait for enough flashing to occur
	return	

	StateSet	RETRIS_GSTATE,2	; prepare to go back to check for more filled lines

	incf		RETRIS_LEVEL	; level increases for each filled row
	ScoreAddLevel			; and increase the score for the level removed

	call		FilledRowCheck	; get that filled line again into W (ignore return)

	call		FilledRowRemove	; get rid of the filled line and shift things down

	goto		ClearDisplayCheck	; check for a clear display (tail return)

RetrisG4state:				;; end of game state, allows the bad piece to blink for awhile
	call		RetrisSpeed
	movwf		RETRIS_SPEED
	StateNext	RETRIS_GSTATE
	;; no "return" here to save space, just falls through to the next state

RetrisG5state:
	decfsz		RETRIS_SPEED
	return
	call		RetrisSpeed
	movwf		RETRIS_SPEED
	StateNext	RETRIS_GSTATE
	;; no "return" here to save space, just falls through to the next state

RetrisG6state:
	decfsz		RETRIS_SPEED
	return
	MusicScorePlay	Warble		;; throw a warble in there
	call		RetrisSpeed
	movwf		RETRIS_SPEED
	StateNext	RETRIS_GSTATE
	;; no "return" here to save space, just falls through to the next state

RetrisG7state:
	decfsz		RETRIS_SPEED
	return
	call		RetrisSpeed
	movwf		RETRIS_SPEED
	StateNext	RETRIS_GSTATE
	;; no "return" here to save space, just falls through to the next state

RetrisG8state:
	decfsz		RETRIS_SPEED
	return
	incf		RETRIS_DONE	;; sets the DONE state
	return

;;;
;;; PieceHitBottom() - goto'd whenever a downward movement can't be accomplished,
;;;			which happens when either (1) the piece hits the bottom
;;;			of the screen, or (2) when it runs into other pieces.
;;;			When a piece "hits bottom" it needs to be checked to see
;;;			if part of it is still above the screen.  If so, the game
;;;			is over.
;;;			This routine is "goto'd" because it sets a completely
;;;			new state and does the high-level return in any case.
;;;
PieceHitBottom:

	DotProcess	bottom,RETRIS_PIECE,RETRIS_POS,MAXX,MAXY
	skpnz
	goto		PieceBottomEnd	; Z was set, a dot was out of bounds - ends the game!
	DotProcessNext	bottom		;   otherwise loop around for the next dot

	;; if we are here, then we bit bottom WITHOUT any part of a piece above the display
	;; which means we just need to move on to the next piece

	StateSet	RETRIS_GSTATE,2		; (set this to "0" to make it skip filled row check)
	goto		PieceDisplay		; tail return

PieceBottomEnd:		;; the end of the game is nigh
			;; we just need to indicate that a new piece didn't fit by drawing
			;; it flashing, and letting it flash for awhile before showing the score

	movlw		2
	call		BlinkSetting

	movlw		0xff		; force the piece down one, and display it flashing
	addwf		RETRIS_POS
	call		PieceDisplay

	StateSet	RETRIS_GSTATE,4
	movlw		0		; always maintain a zero blink environment
	goto		BlinkSetting	; tail return

;;;
;;; RetrisButtons() - process the button presses.  Simply moves a piece left or right.
;;;			This routine gets call fairly deep in the stack, so it is
;;;			written to be long but consume less stack space.  Note that
;;;			only one button is noticed per call...preferring the left.
;;;
RetrisButtons:
	ButtonSkipIfNotPressed	0
	goto	RetrisLeft

	ButtonSkipIfNotPressed	1
	goto	RetrisRight

	ButtonAck		0
	ButtonAck		1

	return

RetrisLeft:
	ButtonAck		0
	call	PieceClear
	call	PieceMoveLeft
	ScorePieceMove			; subtract points for each move
	goto	PieceDisplay		; tail return

RetrisRight:
	ButtonAck		1
	call	PieceClear
	call	PieceMoveRight
	ScorePieceMove			; subtract points for each move
	goto	PieceDisplay		; tail return


;;; ========================================================================
;;; Utility routines below
;;; ========================================================================

;;;
;;; ComposeXY() - MACRO - compose an (x,y) from RETRIS_COL and RETRIS_ROW
;;;			suitable for the display routines, and put it into W.
;;;
ComposeXY:	macro
		swapf	RETRIS_COL,W
		iorwf	RETRIS_ROW,W
		endm

;;;
;;; FilledRowCheck() - checks to see if any rows are "full".  Starts from
;;;			the bottom (0,0) and moves up.  The first filled
;;;			row is returned as an index 0 through MAXY in W.
;;;			Z is set (FALSE) of NO rows are filled
;;;
;;;		Uses - 	WORK0 as the row index
;;;			WORK1 as the column index
;;;			WORK2
;;;
FilledRowCheck:
	clrf	RETRIS_ROW	; used as the row index (starts from bottom)

FilledRowCheck2:		; OUTER loop

	clrf	RETRIS_COL	; used as the column index (starts from left)

FilledRowCheck3:		; INNER loop

	ComposeXY

	call	CheckBit	; sets Z if the bit IS set
	skpz
	goto	FilledRowCheck5	; a bit in this row NOT set, go on to next row

	movlw	MAXX		; check to see if we just processed the last column
	subwf	RETRIS_COL,W
	skpnz
	goto	FilledRowCheck4	; last bit was set, check the next bit

	incf	RETRIS_COL	; increment to the next column and keep going
	goto	FilledRowCheck3

FilledRowCheck4:
	;; at this point, all bits in the current row WERE set, return the row

	movf	RETRIS_ROW,W	; put the full row in W
	bcf	STATUS,Z	; clear the Z bit (TRUE)
	return

FilledRowCheck5:		; need to move on to next row

	movlw	MAXY		; check to see if we just processed the top row
	subwf	RETRIS_ROW,W
	skpnz
	goto	FilledRowCheck6	; we just processed the last row, so we need to return

	incf	RETRIS_ROW	; increment to the next row
	goto	FilledRowCheck2	; go on to the next row

FilledRowCheck6:
	;; did all of the rows, and didn't find a full one
	bsf	STATUS,Z
	return

;;;
;;; FilledRowFlash() - "flashes" the row given in W.  This is done by simply
;;;			resetting the bits in the row with the flash attribute
;;;			turned on.
;;;
FilledRowFlash:
	movwf	RETRIS_ROW
	clrf	RETRIS_COL

	movlw	2
	call	BlinkSetting

FilledRowFlash2:
	ComposeXY

	call	SetBit

	movlw	MAXX		; check to see if we just processed the last column
	subwf	RETRIS_COL,W
	skpnz
	goto	FilledRowFlash3	; we have just processed the last row, so return

	incf	RETRIS_COL	; increment to the next column and keep going
	goto	FilledRowFlash2

FilledRowFlash3:
	movlw	0		; always maintain a zero blink environment
	call	BlinkSetting
	return


;;;
;;; FilledRowRemove() - removes the row given in W.  All of the dots above the
;;;			row are slid down, and the top row cleared.  This is
;;;			a very weird operation, sliding the specific bits down
;;;			of a column.
;;;
FilledRowRemove:
	movwf	RETRIS_WORK0	; has the target row

	;; we move by column, sliding down each row within the column
	;; starting at row W+1, move W+1 down to W until after doing W+1 = MAXY
	;; then clear the MAXY bit

	clrf	RETRIS_COL

FilledRowRemove2:
	movf	RETRIS_WORK0,W
	movwf	RETRIS_ROW

FilledRowRemove3:
	ComposeXY
	
	call	ClearBit	; clears the current (row,col) - particularly for the flashing

	incf	RETRIS_ROW	; move to the row above
	ComposeXY		; get the coordinates
	decf	RETRIS_ROW	; then move back down

	call	CheckBit	; sets Z if the given bit is set
	skpz
	goto	FilledRowRemove4 ; Z was not set, so the bit being moved down isn't set
	
	ComposeXY		; otherwise set this one
	call	SetBit

FilledRowRemove4:		; move on to the next row

	incf	RETRIS_ROW
	movlw	MAXY
	subwf	RETRIS_ROW,W

	skpz
	goto	FilledRowRemove3	; go around to process this row

	;; we are on the top-most row, so just clear it

	ComposeXY
	call	ClearBit

FilledRowRemove5:		; move on to the next column
	movlw	MAXX
	subwf	RETRIS_COL,W
	skpnz
	return			; we're done at this point

	incf	RETRIS_COL
	goto	FilledRowRemove2

;;;
;;; ClearDisplayCheck() - checks to see if the display was cleared after a row
;;;			  has been filled and eliminated.  If it is clear, a
;;;			  sound is played and the score is incremented.
;;;
ClearDisplayCheck:
	clrf		RETRIS_COL

ClearDisplayCheck2:
	clrf		RETRIS_ROW

ClearDisplayCheck3:
	ComposeXY
	call		CheckBit	; sets Z if the given bit is set
	skpnz
	return				; at least one dot was set, so the screen is NOT clear

	incf	RETRIS_ROW		; try the next row
	movlw	(MAXY+1)		; since we inc'd the ROW, 7 is the stopping point
	subwf	RETRIS_ROW,W
	skpz
	goto	ClearDisplayCheck3

	incf	RETRIS_COL		; try the next column
	movlw	(MAXX+1)		; since we inc'd the COL, 5 is the stopping point
	subwf	RETRIS_COL,W
	skpz
	goto	ClearDisplayCheck2

	;; at this point, we've done all points and none were set

	MusicScorePlay	Siren		;; up and down siren and
	MusicScorePlay	Warble		;; throw a warble in there too
	ScoreAddClear			;; add score for clearing the screen

	return
	
;;; 
;;; PieceGetDot() - given a piece number in ARG0 and a "dot number" in W, return
;;;		 the (x,y) coordinates of that dot.
;;;
;;;		 Note that each piece has 3 dots.  The first dot is always
;;;		 at (0,0) which is the origin for the piece.  The other dots
;;;		 are specified as ADDITIVE to the origin.  So a negative move
;;;		 may be 0xf0 which means (-1,0).  Note that some negative moves
;;;		 are done more intricately.  For example:
;;;			(0,-1) -->  0xff
;;;		 In this example, we are going down on the Y coordinate, so we
;;;		 give it an F to add, though this rolls over on the X, so we ensure
;;;		 that X ends up at "0" by making it an F too.
;;;		 Here's the complete table of moves:
;;;
;;;	Coords	Move	Coords	Move	Coords	Move	Coords	Move	Coords	Move
;;;	------------	------------	------------	------------	------------
;;;	(0,0)	0x00	(1,0)	0x10	(2,0)	0x20	(-1,0)	0xf0	(-2,0)	0xe0
;;;	(0,1)	0x01	(1,1)	0x11	(2,1)	0x21	(-1,1)	0xf1	(-2,1)	0xe1
;;;	(0,2)	0x02	(1,2)	0x12	(2,2)	0x22	(-1,2)	0xf2	(-2,2)	0xe2
;;;	(0,-1)	0xff	(1,-1)	0x0f	(2,-1)	0x1f	(-1,-1)	0xef	(-2,-1)	0xdf
;;;	(0,-2)	0xfe	(1,-2)	0x0e	(2,-2)	0x1e	(-1,-2)	0xee	(-2,-2)	0xde
;;;			
	
PieceGetDot:
	movwf	SAVEW		; save the dot number
	andlw	0xff		; check W for a zero (movwf doesn't set Z)
	skpnz
	retlw	0x00		; if the dot number is zero, we ALWAYS return (0,0)

	decf	SAVEW		; adjust the dot number for the PieceTable

	bcf	STATUS,C
	rlf	ARG0		; need to double the piece index
	
	movlw	HIGH PieceTable
	movwf	PCLATH
	movlw	LOW PieceTable	; W has the initial offset to use

	addwf	ARG0,W		; add in the piece index
	skpnc
	incf	PCLATH		; possibly increase high bits if we rolled over

	addwf	SAVEW,W		; add in the dot number
	skpnc
	incf	PCLATH		; again, increase high bits if we rolled over

	bcf	STATUS,C
	rrf	ARG0		; put it back where it belongs
	
	movwf	PCL		; W now has the offset with PCLATH set correctly
	
;;;
;;; PieceTable() - the table that details the dots to be "lit-up" for each
;;;			piece.
;;;		- Each piece has up to three dots
;;;		- Each piece is described with relative coordinates where (0,0)
;;;			is the "origin" of the piece
;;;		This table will return the constant NODATA when there is no
;;;		more dots for the given piece.  This constant is chosen to
;;;		NEVER be a valid dot.
;;;
;;;		The table is currently configured to have the original (first
;;;		dot) be the "lowest" dot so that when it enters the screen it
;;;		takes up only one row.  This really cuts down on the number of
;;;		possible coordinate points!  Though that really doesn't matter.
;;;
;;;		Note that the table is 16 positions long, although many of the
;;;		pieces are repeated in order to adjust the probabilities.

PieceSingle:	macro
		;; singleton
		retlw	NODATA		; (none)	O
		retlw	NODATA		; (none)
		endm

PieceLongVert:	macro
		;; long stick vertical	; 		o
		retlw	0x01		; (0,1)		o
		retlw	0x02		; (0,2)		O 
		endm

PieceLongHoriz:	macro
		;; long stick horiz
		retlw	0xf0		; (-1,0)	oOo
		retlw	0x10		; (1,0)
		endm

PieceShortVert:	macro
		;; 2-er stick vertical
		retlw	0x01		; (0,1)		o
		retlw	NODATA		; (none)	O
		endm

PieceShortHoriz:	macro
		;; 2-er stick horiz
		retlw	0x10		; (1,0)		Oo
		retlw	NODATA		; (none)
		endm

PieceL1:	macro
		;; L-shape rotation 0
		retlw	0x01		; (0,1)		o
		retlw	0x10		; (1,0)		Oo
		endm

PieceL2:	macro
		;; L-shape rotation 1
		retlw	0xf0		; (-1,0)	 o
		retlw	0x01		; (0,1)		oO
		endm

PieceL3:	macro
		;; L-shape rotation 2
		retlw	0x01		; (0,1)		oo
		retlw	0x11		; (1,1)		O
		endm

PieceL4:	macro
		;; L-shape rotation 3
		retlw	0x01		; (0,1)		oo
		retlw	0xf1		; (-1,1)	 O
		endm
PieceTable:
		PieceSingle	; 0
		PieceSingle	; 1 - 1/8 probably for a single
		PieceLongVert	; 2 - 1/16 for a long vert
		PieceLongHoriz	; 3 - 1/16 for a long horiz
		PieceShortVert	; 4 - 3/16 for a short vert
		PieceShortVert	; 5
		PieceShortVert	; 6 - 3/16 for a short vert
		PieceShortHoriz	; 7
		PieceShortHoriz	; 8
		PieceShortHoriz	; 9 - 3/16 for a short horiz
		PieceL1		; 10/A
		PieceL4		; 11/B - 1/16 for an L1
		PieceL2		; 12/C - 1/16 for an L2
		PieceL3		; 13/D
		PieceL3		; 14/E - 1/8 for an L3
		PieceL4		; 15/F - 1/8 for an L4


;;;
;;; PieceCheck() - given a screen coordinate in W, try to position the
;;;		 given piece (in ARG0) at that screen coordinate.  This routine
;;;		 sets Z for FALSE if the try failed.  WORK0 and WORK1 are used.
;;;
;;;		 The rules for trying are as follows:
;;;
;;;		- any dots that would otherwise be drawn ABOVE the top of
;;;		  the screen are ignored - ie they don't figure into the
;;;		  "try" - those dots always "work"
;;;
;;;		- if a dot is to be drawn on a point where there is already
;;;		  a dot, this routine returns "FALSE" (see above)
;;;
;;;		- if a dot is to be drawn below the bottom of the screen
;;;		  this routine returns "FALSE" (see above)
;;;
;;;		- if a dot is to be drawn to the left or right of the edges
;;;		  of the screen this routine returns "FALSE" (see above)
;;;
;;;		- it is ASSUMED that the screen coordinate given is valid
;;;

#define MAXY_EXTRA	(MAXY+4)
	
PieceCheck:
	movwf		RETRIS_TARGET	; the target screen coordinates

	DotProcess	check,ARG0,RETRIS_TARGET,MAXX,MAXY_EXTRA
	skpnz
	goto		PieceCheckBad	; if Z is set, then the piece was out of bounds

	;; For this routine, we have to check also if the given bit is set.
	;; This, however, could possibly check a Y value that is above the screen
	;; so Y needs to be re-checked ahead of time.

	movwf		SAVEW	; save it for later
	andlw		0x0f	; mask off X bits
	sublw		MAXY	; subtract target Y from MAXY - setting C if in bounds
	movf		SAVEW,W	; restore W

	;; at this point C is set of the dot is within the screen display
	;; note that it needs to be checked if so

	bcf		STATUS,Z	; clears Z, which defaults the screen bit as off
	btfsc		STATUS,C	; skips the next instruction if the dot was out of screen bounds
	call		CheckBit
	skpnz
	goto		PieceCheckBad	; if Z is set, the bit is set, so we can't move there

	;; so this dot is good, go on to the next one

	DotProcessNext	check

	;; all dots were good, so we can return 

	bcf		STATUS,Z	; clear the Z to indicate "TRUE", all points work
	return

PieceCheckBad:
	bsf		STATUS,Z	; set the Z indicating "FALSE", piece will not work
	return
	
;;;
;;; PieceMoveDown() - move the given piece down one level.  The CURRENT
;;;			position is given in W and the piece in ARG0.
;;; 
;;;			If the piece can't be moved down, "FALSE" is returned
;;;			- the Z bit is set.  If Z is clear "TRUE", then the
;;;			piece was moved down as requested.
;;;
;;;			Note that the position will be increased to Y==-1
;;;			which causes even a singleton to stop.
;;;
;;;			NOTE - seems to be a problem with my thinking about
;;;			adding 0xff to get Y-1 - doesn't seem to work on 0x00.
;;;			Which is an important point, we shouldn't ever try
;;;			to have an absolute origin position that is less than (0,0)

PieceMoveDown:
	movf	RETRIS_PIECE,W
	movwf	ARG0
	
	movf	RETRIS_POS,W
	andlw	0x0f		; mask off bits to check Y
	skpnz
	goto	PieceMoveDownNot ; if already at zero on the Y, can't move down further
	
	movf	RETRIS_POS,W
	addlw	0xff		; decreases Y by one, and leaves X

	call	PieceCheck	; W has attempted position, ARG0 has piece, try to move the piece down one
	skpnz
	goto	PieceMoveDownNot

PieceMoveDownOK:
	decf	RETRIS_POS	; it is OK to simply decrement the position at this point (it hits only Y)
	bcf	STATUS,Z	; a clear Z indicates success
	return
	
PieceMoveDownNot:
	bsf	STATUS,Z
	return

;;;
;;; PieceMoveLeft() - move the given piece to the left one dot.  The CURRENT
;;;			position is given in ARG0 and the piece in W.
;;; 
;;;			If the piece can't be moved left, "FALSE" is returned
;;;			- the Z bit is set.  If Z is clear "TRUE", then the
;;;			piece was moved left as requested.
;;;
PieceMoveLeft:	
	movf	RETRIS_PIECE,W
	movwf	ARG0
	
	swapf	RETRIS_POS,W	; swap X and Y to check only X
	andlw	0x0f		; mask off bits other bits
	skpnz
	goto	PieceMoveLeftNot ; if already at zero on the X, can't move left further
	
	movf	RETRIS_POS,W
	addlw	0xf0		; decreases X by one, and leaves Y

	call	PieceCheck	; W has attempted position, ARG0 has piece, try to move the piece
	skpnz
	goto	PieceMoveLeftNot

PieceMoveLeftOK:
	movlw	0xf0
	addwf	RETRIS_POS	; decrement X, it's OK
	bcf	STATUS,Z	; a clear Z indicates success
	return
	
PieceMoveLeftNot:
	bsf	STATUS,Z
	return
	
;;;
;;; PieceMoveRight() - move the given piece to the right one dot.  The CURRENT
;;;			position is given in ARG0 and the piece in W.
;;; 
;;;			If the piece can't be moved right, "FALSE" is returned
;;;			- the Z bit is set.  If Z is clear "TRUE", then the
;;;			piece was moved right as requested.
;;;
PieceMoveRight:	
	movf	RETRIS_PIECE,W
	movwf	ARG0
	
	swapf	RETRIS_POS,W	; swap X and Y to check only X
	andlw	0x0f		; mask off bits other bits
	xorlw	MAXX		; check against max
	skpnz
	goto	PieceMoveRightNot ; if already at MAXX can't move any farther
	
	movf	RETRIS_POS,W
	addlw	0x10		; increases X by one, and leaves Y

	call	PieceCheck	; W has attempted position, ARG0 has piece, try to move the piece
	skpnz
	goto	PieceMoveRightNot

PieceMoveRightOK:
	movlw	0x10
	addwf	RETRIS_POS	; increment X, it's OK
	bcf	STATUS,Z	; a clear Z indicates success
	return
	
PieceMoveRightNot:
	bsf	STATUS,Z
	return


;;;
;;; PieceNew() - start a new piece.  The piece is randomly selected and positioned
;;;		with the current position as just above top center.  In this way
;;;		it can be moved down, which checks for collision easily.
;;;
PieceNew:
	movlw	0x27		; middle of row ABOVE the top
	movwf	RETRIS_POS

	movf	RANDOM,W	; get a random piece
	andlw	0x0f		; mod 16
	movwf	RETRIS_PIECE
	return

;;;
;;; PieceDisplay() - PieceClear()
;;;		Draw or Clear the current piece.  Any blinking needs to be set ahead of time.
;;;		A piece CAN be drawn or cleared even though part of it may be off-screen.
;;;		Those dots simply aren't drawn/cleared.
;;;
;;;	RETRIS_POS - current position of the current piece
;;;	RETRIS_PIECE - the current piece (index into piece table)
;;;
PieceDisplay:

	DotProcess	display,RETRIS_PIECE,RETRIS_POS,MAXX,MAXY

	;; Z will be set (FALSE) if the dot in W is outside of the given bounds
	;;   Z will be clear (TRUE) otherwise
	;; for this routine, only call the SetBit routine for points within bounds

	skpz
	call	SetBit		; only called if Z not set (TRUE) by DotProcess

	DotProcessNext	display	; loop around for the next dot

	return

;;;
;;; PieceClear() - clear the current piece - just the reverse of drawing.
;;;
PieceClear:
	DotProcess	clear,RETRIS_PIECE,RETRIS_POS,MAXX,MAXY

	;; Z will be set (FALSE) if the dot in W is outside of the given bounds
	;;   Z will be clear (TRUE) otherwise
	;; for this routine, only call the ClearBit routine for points within bounds

	skpz
	call	ClearBit	; only called if Z not set (TRUE) by DotProcess

	DotProcessNext	clear	; loop around for the next dot

	return

;;;
;;; RetrisSpeed() - returns the delay loop which ranges according to the number
;;;			of lines (RETRIS_LEVEL) that are destroyed.  Once it
;;;			gets up to a high level, it becomes simply impossible.
;;;
;;;			There are 16 speeds.  These USED to correspond to the
;;;			upper bits of the level.  But the level only increases
;;;			upon each eliminated row, so making such that 240 or
;;;			so rows need to be eliminated for the highest speed
;;;			just didn't make sense.
;;;
;;;			Now, the speed is based upon the 2nd through 5th
;;;			bits.  So a new level starts every 2 eliminated rows.
;;;			Yes, it speeds up rather quickly.  Top speed is
;;;			reached after 32 eliminated rows...which is really
;;;			quick a few rows.
;;;
RetrisSpeed:

;;; old "upper bits" approach
;;;	swapf	RETRIS_LEVEL,W		; get the high bits of the level

;;; new approach
	rrf	RETRIS_LEVEL,W		; slide the level right 1 place
	
	andlw	0x0f			; Mask off all but the target 4 bits
	movwf	SAVEW

	movlw	HIGH RetrisSpeedTable
	movwf	PCLATH
	movlw	LOW RetrisSpeedTable
	addwf	SAVEW,W
	skpnc
	incf	PCLATH
	movwf	PCL

RetrisSpeedTable:
	retlw	d'250'	; speed 0
	retlw	d'240'	; speed 1
	retlw	d'230'	; speed 2
	retlw	d'220'	; speed 3
	retlw	d'210'	; speed 4
	retlw	d'200'	; speed 5
	retlw	d'190'	; speed 6
	retlw	d'180'	; speed 7
	retlw	d'170'	; speed 8
	retlw	d'160'	; speed 9
	retlw	d'150'	; speed 10
	retlw	d'140'	; speed 11
	retlw	d'130'	; speed 12
	retlw	d'120'	; speed 13
	retlw	d'110'	; speed 14
	retlw	d'100'	; speed 15

;;
;; The list file for this puppy gets HUGE because of macro expansion.
;; This line fixes that.
;;
	noexpand

Siren:
	MusicNoteTableEntry	"C",   ,4,0,0
	MusicNoteTableEntry	"C","#",4,0,0
	MusicNoteTableEntry	"D",   ,4,0,0
	MusicNoteTableEntry	"E","b",4,0,0
	MusicNoteTableEntry	"E",   ,4,0,0
	MusicNoteTableEntry	"F",   ,4,0,0
	MusicNoteTableEntry	"F","#",4,0,0
	MusicNoteTableEntry	"G",   ,4,0,0
	MusicNoteTableEntry	"A","b",4,0,0
	MusicNoteTableEntry	"A",   ,4,0,0
	MusicNoteTableEntry	"B","b",4,0,0
	MusicNoteTableEntry	"B",   ,4,0,0
	MusicNoteTableEntry	"C",   ,5,0,0
	MusicNoteTableDone

Welcome:
	dt	" RETRIS 2007 - ",0

CountDown:
	dt	"3 2 1",0
	
ScoreLeadin:
	dt	" --",0

;;; =======================================================================
;;; =======================================================================

	END
