;;;
;;; score.asm
;;;	The BCD conversion code in this file was found
;;;	on the web site below.  The code included this copyright:
;;; 
;;;   ***************************************************************
;;;   * Copyright (C) 2006, Embed Inc (http://www.embedinc.com)     *
;;;   *                                                             *
;;;   * Permission to copy this file is granted as long as this     *
;;;   * copyright notice is included in its entirety at the         *
;;;   * beginning of the file, whether the file is copied in whole  *
;;;   * or in part and regardless of whether other information is   *
;;;   * added to the copy.                                          *
;;;   *                                                             *
;;;   * The contents of this file may be used in any way,           *
;;;   * commercial or otherwise.  This file is provided "as is",    *
;;;   * and Embed Inc makes no claims of suitability for a          *
;;;   * particular purpose nor assumes any liability resulting from *
;;;   * its use.                                                    *
;;;   ***************************************************************
;;;
;;; the following is code for a score test in main - just change the #'s for testing
;;;
;;;	;;; DEBUG
;;;	;;; score test
;;;
;;;	movlw	0x00
;;;	movwf	SCORE_LOW
;;;	movlw	0x00
;;;	movwf	SCORE_HIGH
;;;	call	ScoreConvert

#include "regdef.h"
#include "portdef.h"

	global	ScoreConvert
	global	ScoreClear
	global	ScoreInit
	global	ScoreInc
	global	ScoreAdd
	global	ScoreSub
	global	ScoreBigAdd
	global	ScoreBigSub
	
PROG1	CODE

;;
;; ScoreInit() - initializes the scoring
;;
ScoreInit:
	call	ScoreClear
	return

;;;
;;; ScoreClear() - clears out the score
;;;
ScoreClear:
	clrf	SCORE_HIGH
	clrf	SCORE_LOW
	return

;;;
;;; ScoreInc() - increments the score by 1
;;;
ScoreInc:
	incf	SCORE_LOW
	skpnz
	incf	SCORE_HIGH
	return

;;;
;;; ScoreAdd() - adds W to the score - can only do 8-bit adds
;;;
ScoreAdd:
	addwf	SCORE_LOW
	skpnc
	incf	SCORE_HIGH
	return

;;;
;;; ScoreBigAdd() - adds a multiple of 256 to the current score
;;;
ScoreBigAdd:
	addwf	SCORE_HIGH
	return

;;;
;;; ScoreSub() - subtracts an 8-bit number from the score.  Stops at zero.
;;;
ScoreSub:
	subwf	SCORE_LOW
	skpnc
	return

	;; we have an underflow here

	movf	SCORE_HIGH
	skpz
	goto	ScoreSubAdjust

	;; at this point we've smacked into zero

	clrf	SCORE_LOW	; underflow past zero, so reset to zero
	return

ScoreSubAdjust:
	decf	SCORE_HIGH
	return

;;
;; ScoreBigSub() - subtracts a multiple of 256 from the score.  Stops at zero.
;;
ScoreBigSub:
	subwf	SCORE_HIGH
	skpc
	clrf	SCORE_HIGH
	return

;;;
;;; ScoreConvert() - convert the current score into BCD for display.
;;;
ScoreConvert:	
	movf	SCORE_HIGH,W
	movwf	ARG1
	movf	SCORE_LOW,W
	movwf	ARG0
	movlw	SCORE_BCD_0
	movwf	FSR
	goto	I16BCD		; this routine does the return

;;;
;;; I16BCD() - converts the given 16-bit number ARG1/ARG0 (high/low)
;;;		into a 5 digit BCD number pointed to by FSR/INDF.
;;;		USES:	 WORK0, WORK1 as loop counters
	
;;; 
I16BCD:	
         movlw   5           ;init number of digits left to generate
         movwf   WORK0
	
;
;********************
;
;   Divide the 16 bit integer value in ARG1:ARG0 by 10.  The remainder is
;   written to where FSR is pointing.  ARG1:ARG0 is updated with the
;   quotient.
;
;   Register usage:
;
;     ARG1:ARG0  -  Starts as numerator.  Quotient is shifted into LSB
;       by one bit each iteration.
;
;     WORK1  -  Loop counter.  Number of iterations left to do.
;
	
I16BCD_Loop:	                   ;back here to generate each new BCD digit

	movlw   d'13'		;init number of iterations left to do
	movwf   WORK1
	bcf     STATUS, C	;init 17th bit to 0
;
;   One quotient bit is produced each iteration of DIV_LOOP.  The 16 bit
;   value numerator is compared to the reference value of 10 * 2**13, the
;   high byte of which is 10100000b with the low byte 0.  If the numerator
;   is greater than or equal to the reference value, then the quotient bit
;   is a 1 and the reference value is removed (subtracted) from the numerator.
;   If it is less than the numerator, then the quotient bit is 0 and the
;   reference value is not subtracted off.
;
;   This pretty much mimics the way long division is done by hand.  Note
;   that if you were dividing 65535 by 10, you would first divide the
;   leading "65" part by 10 to produce a 6 quotient digit.  By only
;   dividing the left few digits of the numerator, you are really shifting
;   the denominator left several digits and doing the divide.  In other
;   words, the first part of the long division of dividing 65535 by 10
;   is really dividing 65535 by 10000.  This yields the high digit of the
;   quotient (6), with the remainder used to generate the remaining digits.
;   The quotient digit times the shifted denominator (60000) is removed
;   from the numerator, and the remainder used as the numerator for the
;   next iteration to get the next digit.  In this example 65535 - 60000
;   = 5535, which is used to get the next quotient digit.  Each time
;   the denominator is shifted right one digit.  Therefore, the next
;   iteration will divide 5535 by 1000.
;
;   The code below uses this same technique, except that it does it in
;   binary instead of base 10.  This means the actual "divide" step is
;   just a compare.  The quotient digit is 1 if the shifted denominator
;   is greater than or equal to the current numerator, and 0 otherwise.
;   Since the quotient digit can only be 0 or 1, it is either subtracted
;   directly from the numerator (quotient digit 1) or not (quotient digit
;   0).
;
;   Instead of shifting the denominator right one digit each iteration,
;   the algorithm below shifts the numerator left one digit (bit) each
;   iteration.  This is equivalent mathematically.  It also frees up
;   the low bit of the numerator each time, which is where the new
;   quotient bit is stored.  After 13 iterations, the low 13 bits of the
;   numerator are the quotient, and the high 3 bits are the remainder
;   which is always a value of 0-9.
;
;   There is one additional wrinkle that may be a bit confusing.  The numerator
;   after the denominator times the quotient bit is removed is always
;   less than denominator, but it may have its high bit set.  When the
;   numerator is shifted left one 1, this high bit must be taken into
;   account.  If this high bit is set at the start of an iteration, then
;   the quotient bit is always 1, since the value with the high bit set
;   is always larger than the denominator.  This high bit is left in C
;   after the numerator is shifted left 1 bit at the end of the previous
;   iteration.  This is referred to in the comments as the "17th" bit,
;   and is checked at the start of each iteration.
;
div_loop:			;back here to find each quotient bit
	movlw	b'10100000'	;get high byte of value 10 shifted left 13 bits
	skpc			;17th bit is set ?
	goto	high0		;no, need to do compare

	subwf	ARG1		;update numerator with denominator removed
	bsf	STATUS, C	;shift in a 1 quotient bit this iteration
	goto	div_shq		;go shift in the quotient bit

high0:				;17th high bit is zero
	subwf	ARG1, W		;compare to remaining value
	skpnc			;this quotient bit is 0, bit value is in C ? (may need skpc)
	movwf	ARG1		;update numerator with denominator removed
div_shq:			;C contains new quotient bit
	rlf	ARG0		;shift numerator left 1, new quotient bit into LSB
	rlf	ARG1
	decfsz	WORK1		;cone one less iteration left to do
	goto	div_loop	;back to make next quotient bit
;
;   The quotient is in the low 13 bits of ARG1:ARG0.  The low 3 bits of the
;   remainder are in the high 3 bits of ARG1:ARG0, and the high bit of the
;   remainder is in C.
;
;   Write the remainder as the next BCD digit value.  FSR is pointing to the
;   BCD digit value byte.
;
	rrf	ARG1, W	;get remainder in bits 7:4
	andlw	b'11110000'	;mask in only the remainder bits
	movwf	INDF		;write to this BCD digit
	swapf	INDF		;put the digit value into the low 3 bits
;
;   Clean up the quotient in ARG1:ARG0.
;
	movlw	b'00011111'	;get mask for valid quotient bits in ARG1
	andwf	ARG1		;mask in only the quotient bits
;
;   Done with the divide.
;
;********************
;
	incf	FSR		;update pointer to next result digit
	decfsz	WORK0		;count one less digit left to generate
	goto	I16BCD_Loop	;back to do next digit

	return

;;; =======================================================================
;;; =======================================================================

	END
	