;;
;; sound.asm
;;
;;   This file implements the sound sub-system for games.
;;
#include "portdef.h"
#include "regdef.h"

	global	SoundUpdate
	global	SoundInit
	global	SoundFreq
	global	SoundNote
	global	SoundOn
	global	SoundOff
	global	SoundDuration
	global	SoundDurationUpdate

#define	SOUND_ON_BIT	SOUND_ON,0x00

;;
;; SoundCheckOnSkip() - MACRO - skips the next intruction is sound is ON
;;
SoundCheckOnSkip:	macro
			btfss	SOUND_ON_BIT
			endm


PROG1	CODE


;;
;; SoundDuration() - turns on the sound for a given period of main loop "ticks"
;;			Requires calling SoundDurationUpdate() in the main loop
;;			Has the effect of turn on the sound too.
;;			W has the sound duration.
;;
SoundDuration:
	movwf	SOUND_DURATION
	call	SoundOn
	return

SoundDurationUpdate:
	SoundCheckOnSkip
	return

	decfsz	SOUND_DURATION
	return

	call	SoundOff
	return

;;
;; SoundUpdate() - routine called by the main loop to update the
;;		   the sound...in other words, cause a wave-form
;;		   to be generated.
;;
SoundUpdate:
	SoundCheckOnSkip
	return		; if the sound is off, then just return

	;; otherwise update the counter

	decfsz	SOUND_STATE
	return		; haven't hit the right frequency yet

	;; at this point, we need to generate the wave form and reset counter

	btfsc	SOUND_PORT
	goto	SoundTop

SoundBot:
	bsf	SOUND_PORT	; it was a zero, toggle it
	goto	SoundReset

SoundTop:
	bcf	SOUND_PORT	; it was a one, toggle it

SoundReset:
	movf	SOUND_FREQ,W
	movwf	SOUND_STATE
	return


;;
;; SOUND_FREQ - register that contains the current sound frequency
;;		which is specified as the number of sound "ticks" to
;;		wait for each 1/2 wave.  Zero means that each call
;;		to the sound update routine will flip the wave.
;;
;; SOUND_ON - if non-zero, then the sound is "on", meaning that the
;;		wave form will be produced.
;;
;; SOUND_STATE - stores the current count on the way to given Freq.
;;

;;
;; SoundInit() - initializes the sound sub-system
;;
SoundInit:
	call	SoundOff
	movlw	1
	call	SoundFreq
	return

;;
;; SoundOn/SoundOff - turns on/off the sound subsystem
;;
SoundOff:
	bcf	SOUND_ON_BIT
	return

SoundOn:
	bsf	SOUND_ON_BIT
	return

;;
;; SoundFreq - given a freq selector, set up for that particular frequency
;;
SoundFreq:
	movwf	SOUND_FREQ
	movwf	SOUND_STATE
	return

;;
;; SoundNote() - pre-programmed notes for sound, just give it a W
;;
SoundNote:
	addlw	d'1'			; make sure it is not zero (minimum of 8)
	bcf	STATUS,C
	movwf	SOUND_FREQ
	movwf	SOUND_STATE
	rlf	SOUND_FREQ
	rlf	SOUND_FREQ
	rlf	SOUND_FREQ
	rlf	SOUND_FREQ
	rlf	SOUND_FREQ
	rlf	SOUND_FREQ
	return

;;; =======================================================================
;;; =======================================================================

	END
