;;
;; textdata.asm
;;	This file contains the data for characters and code to implement
;;	the jump table to return the appropriate scan line for a given char/num.
;;
#include "portdef.h"
#include "regdef.h"

	global	TextASCII

PROG1	CODE

;;; This routine has been changed to be simpler, but bigger.  It now implements
;;; a complete subset of the ascii table.  Some characters may not be filled in (well).
;;; But the table is complete for the characters.  Note that it is split into two
;;; halfs so that the table isn't bigger than 256 slots (since each char has 4 scan lines)
;;
;; Some of the more "useless" characters are mapped to more interesting characters to make
;; some interesting effects possible:
;;
;;	# - inverse of a space
;;	% - 
;;	& - 
;;	@ - 
;;	^ - 
;;	` - 
;;;
;;;    040   32    20    SPACE    120   80    50    P
;;;    041   33    21    !        121   81    51    Q
;;;    042   34    22    "        122   82    52    R
;;;    043   35    23    #        123   83    53    S
;;;    044   36    24    $        124   84    54    T
;;;    045   37    25    %        125   85    55    U
;;;    046   38    26    &        126   86    56    V
;;;    047   39    27    '        127   87    57    W
;;;    050   40    28    (        130   88    58    X
;;;    051   41    29    )        131   89    59    Y
;;;    052   42    2A    *        132   90    5A    Z
;;;    053   43    2B    +        133   91    5B    [
;;;    054   44    2C    ,        134   92    5C    \
;;;    055   45    2D    -        135   93    5D    ]
;;;    056   46    2E    .        136   94    5E    ^
;;;    057   47    2F    /        137   95    5F    _
;;;    060   48    30    0        140   96    60    `
;;;    061   49    31    1        141   97    61    a
;;;    062   50    32    2        142   98    62    b
;;;    063   51    33    3        143   99    63    c
;;;    064   52    34    4        144   100   64    d
;;;    065   53    35    5        145   101   65    e
;;;    066   54    36    6        146   102   66    f
;;;    067   55    37    7        147   103   67    g
;;;    070   56    38    8        150   104   68    h
;;;    071   57    39    9        151   105   69    i
;;;    072   58    3A    :        152   106   6A    j
;;;    073   59    3B    ;        153   107   6B    k
;;;    074   60    3C    <        154   108   6C    l
;;;    075   61    3D    =        155   109   6D    m
;;;    076   62    3E    >        156   110   6E    n
;;;    077   63    3F    ?        157   111   6F    o
;;;    100   64    40    @        160   112   70    p
;;;    101   65    41    A        161   113   71    q
;;;    102   66    42    B        162   114   72    r
;;;    103   67    43    C        163   115   73    s
;;;    104   68    44    D        164   116   74    t
;;;    105   69    45    E        165   117   75    u
;;;    106   70    46    F        166   118   76    v
;;;    107   71    47    G        167   119   77    w
;;;    110   72    48    H        170   120   78    x
;;;    111   73    49    I        171   121   79    y
;;;    112   74    4A    J        172   122   7A    z
;;;    113   75    4B    K
;;;    114   76    4C    L
;;;    115   77    4D    M
;;;    116   78    4E    N
;;;    117   79    4F    O

;;
;; TextASCII() - front-end to Char/Num that does the encoding for ASCII
;;		Notes 'space' as well.  W holds the line, ARG0 the char.
;;		Uses WORK0, WORK1.  ARG0 is not changed.

TextASCII:
	movwf	WORK0		; WORK0 now holds the target scan line
	movlw	0x20		; code for a space, subtract it right away
	subwf	ARG0,W
	movwf	WORK1		; and WORK1 holds the character where space == zero

	;; now we need to do the calculations for the jump table, note however that
	;; we have 94 characters in the table, and we need to multiply that by 4
	;; which is over 256, meaning that we may need to increment PCLATH up to twice

	;; first, double the character to use as a double addition

	bcf	STATUS,C	; clear the carry just in case
	rlf	WORK1		; now WORK1 has the character *2

	movlw	HIGH CharTable	; load up the high bits of the table
	movwf	PCLATH

	movlw	LOW CharTable	; and the low bits

	;; now we need to calculate the jump point for the table

	addwf	WORK1,W		; add in char*2
	skpnc
	incf	PCLATH

	;; at this point W points to the char*2 entry, our target is char*4+scanline

	addwf	WORK1,W		; add in char*2 again
	skpnc
	incf	PCLATH

	;; almost there, W has char*4, need to add in the scan line

	addwf	WORK0,W
	skpnc
	incf	PCLATH

	movwf	PCL		; and dispatch into the table

;;
;; CharTable - a table of scan lines, 4 for each character.
;;		Note that characters are given in "rev video" :-)
;;		Seven bits per line.  Top of char to the left,
;;		to keep the bottom bits as bit 0,
;;		and the characters are mirrored so that the left
;;		side comes out first.
;;
;;		Note that some have a blank trailing line, these
;;		should be noticed and cause the cell to be 3 lines
;;		instead of 4.
;;
CharTable:
CharSpace:
	retlw	b'1111111'
	retlw	b'1111111'
	retlw	b'1111111'
	retlw	b'1111111'
CharBang:
	retlw	b'1111111'
	retlw	b'1000101'
	retlw	b'1111111'
	retlw	b'1111111'
CharDQuote:
	retlw	b'1001111'
	retlw	b'1111111'
	retlw	b'1001111'
	retlw	b'1111111'
CharPound:
	retlw	b'0000000'
	retlw	b'0000000'
	retlw	b'0000000'
	retlw	b'0000000'
CharDollar:
	retlw	b'1111111'
	retlw	b'1111111'
	retlw	b'1111111'
	retlw	b'1111111'
CharPercent:
	retlw	b'1111111'
	retlw	b'1111111'
	retlw	b'1111111'
	retlw	b'1111111'
CharAnd:
	retlw	b'1111111'
	retlw	b'1111111'
	retlw	b'1111111'
	retlw	b'1111111'
CharSQuote:
	retlw	b'1111111'
	retlw	b'1001111'
	retlw	b'1111111'
	retlw	b'1111111'
CharLParen:
	retlw	b'1100011'
	retlw	b'1011101'
	retlw	b'1111111'
	retlw	b'1111111'
CharRParen:
	retlw	b'1011101'
	retlw	b'1100011'
	retlw	b'1111111'
	retlw	b'1111111'
CharStar:
	retlw	b'1111111'
	retlw	b'1111111'
	retlw	b'1111111'
	retlw	b'1111111'
CharPlus:
	retlw	b'1110111'
	retlw	b'1100011'
	retlw	b'1110111'
	retlw	b'1111111'
CharComma:
	retlw	b'1111110'
	retlw	b'1111001'
	retlw	b'1111111'
	retlw	b'1111111'
CharDash:
	retlw	b'1110111'
	retlw	b'1110111'
	retlw	b'1110111'
	retlw	b'1111111'
CharPeriod:
	retlw	b'1111111'
	retlw	b'1111101'
	retlw	b'1111111'
	retlw	b'1111111'
CharSlash:
	retlw	b'1001111'
	retlw	b'1110111'
	retlw	b'1111001'
	retlw	b'1111111'
Num0:
	retlw	b'1000001'
	retlw	b'1011101'
	retlw	b'1000001'
	retlw	b'1111111'
Num1:
	retlw	b'1101101'
	retlw	b'1000001'
	retlw	b'1111101'
	retlw	b'1111111'
Num2:
	retlw	b'1010001'
	retlw	b'1010101'
	retlw	b'1000101'
	retlw	b'1111111'
Num3:
	retlw	b'1011101'
	retlw	b'1010101'
	retlw	b'1000001'
	retlw	b'1111111'
Num4:
	retlw	b'1000111'
	retlw	b'1110111'
	retlw	b'1000001'
	retlw	b'1111111'
Num5:
	retlw	b'1000101'
	retlw	b'1010101'
	retlw	b'1010001'
	retlw	b'1111111'
Num6:
	retlw	b'1000001'
	retlw	b'1010101'
	retlw	b'1010001'
	retlw	b'1111111'
Num7:
	retlw	b'1011001'
	retlw	b'1010111'
	retlw	b'1001111'
	retlw	b'1111111'
Num8:
	retlw	b'1000001'
	retlw	b'1010101'
	retlw	b'1000001'
	retlw	b'1111111'
Num9:
	retlw	b'1000101'
	retlw	b'1010101'
	retlw	b'1000001'
	retlw	b'1111111'
CharColon:
	retlw	b'1111111'
	retlw	b'1110101'
	retlw	b'1111111'
	retlw	b'1111111'
CharSColon:
	retlw	b'1111110'
	retlw	b'1110101'
	retlw	b'1111111'
	retlw	b'1111111'
CharLess:
	retlw	b'1110111'
	retlw	b'1101011'
	retlw	b'1011101'
	retlw	b'1111111'
CharEqual:
	retlw	b'1101011'
	retlw	b'1101011'
	retlw	b'1101011'
	retlw	b'1111111'
CharGreater:
	retlw	b'1011101'
	retlw	b'1101011'
	retlw	b'1110111'
	retlw	b'1111111'
CharQuestion:
	retlw	b'1011111'
	retlw	b'1010010'
	retlw	b'1101111'
	retlw	b'1111111'
CharAt:
	retlw	b'1111111'
	retlw	b'1111111'
	retlw	b'1111111'
	retlw	b'1111111'
CharA:
	retlw	b'1100001'
	retlw	b'1010111'
	retlw	b'1100001'
	retlw	b'1111111'
CharB:
	retlw	b'1000001'
	retlw	b'1010101'
	retlw	b'1101011'
	retlw	b'1111111'
CharC:
	retlw	b'1100011'
	retlw	b'1011101'
	retlw	b'1011101'
	retlw	b'1111111'
CharD:
	retlw	b'1000001'
	retlw	b'1011101'
	retlw	b'1100011'
	retlw	b'1111111'
CharE:
	retlw	b'1000001'
	retlw	b'1010101'
	retlw	b'1011101'
	retlw	b'1111111'
CharF:
	retlw	b'1000001'
	retlw	b'1010111'
	retlw	b'1011111'
	retlw	b'1111111'
CharG:
	retlw	b'1100011'
	retlw	b'1011101'
	retlw	b'1010101'
	retlw	b'1010011'
CharH:
	retlw	b'1000001'
	retlw	b'1110111'
	retlw	b'1000001'
	retlw	b'1111111'
CharI:
	retlw	b'1011101'
	retlw	b'1000001'
	retlw	b'1011101'
	retlw	b'1111111'
CharJ:
	retlw	b'1111011'
	retlw	b'1011101'
	retlw	b'1000001'
	retlw	b'1011111'
CharK:
	retlw	b'1000001'
	retlw	b'1100111'
	retlw	b'1011011'
	retlw	b'1111101'
CharL:
	retlw	b'1000001'
	retlw	b'1111101'
	retlw	b'1111101'
	retlw	b'1111111'
CharM:
	retlw	b'1000001'
	retlw	b'1101111'
	retlw	b'1101111'
	retlw	b'1000001'
CharN:
	retlw	b'1000001'
	retlw	b'1101111'
	retlw	b'1111011'
	retlw	b'1000001'
CharO:
	retlw	b'1100011'
	retlw	b'1011101'
	retlw	b'1100011'
	retlw	b'1111111'
CharP:
	retlw	b'1000001'
	retlw	b'1010111'
	retlw	b'1101111'
	retlw	b'1111111'
CharQ:
	retlw	b'1100011'
	retlw	b'1011101'
	retlw	b'1010101'
	retlw	b'1100001'
CharR:
	retlw	b'1000001'
	retlw	b'1010111'
	retlw	b'1101001'
	retlw	b'1111111'
CharS:
	retlw	b'1000101'
	retlw	b'1010101'
	retlw	b'1010001'
	retlw	b'1111111'
CharT:
	retlw	b'1011111'
	retlw	b'1000001'
	retlw	b'1011111'
	retlw	b'1111111'
CharU:
	retlw	b'1000001'
	retlw	b'1111101'
	retlw	b'1000001'
	retlw	b'1111111'
CharV:
	retlw	b'1000011'
	retlw	b'1111101'
	retlw	b'1000011'
	retlw	b'1111111'
CharW:
	retlw	b'1000001'
	retlw	b'1111011'
	retlw	b'1111011'
	retlw	b'1000001'
CharX:
	retlw	b'1011101'
	retlw	b'1100111'
	retlw	b'1011101'
	retlw	b'1111111'
CharY:
	retlw	b'1001111'
	retlw	b'1110001'
	retlw	b'1001111'
	retlw	b'1111111'
CharZ:
	retlw	b'1011001'
	retlw	b'1010101'
	retlw	b'1001101'
	retlw	b'1111111'
CharLSquare:
	retlw	b'1000001'
	retlw	b'1011101'
	retlw	b'1111111'
	retlw	b'1111111'
CharBSlash:
	retlw	b'1111001'
	retlw	b'1110111'
	retlw	b'1001111'
	retlw	b'1111111'
CharRSquare:
	retlw	b'1011101'
	retlw	b'1000001'
	retlw	b'1111111'
	retlw	b'1111111'
CharCaret:
	retlw	b'1101111'
	retlw	b'1011111'
	retlw	b'1101111'
	retlw	b'1111111'
CharUL:
	retlw	b'1111110'
	retlw	b'1111110'
	retlw	b'1111110'
	retlw	b'1111110'
CharBackTick:
	retlw	b'1111111'
	retlw	b'1101111'
	retlw	b'1011111'
	retlw	b'1111111'
Chara:
	retlw	b'1110001'
	retlw	b'1110101'
	retlw	b'1111011'
	retlw	b'1111111'
Charb:
	retlw	b'1000001'
	retlw	b'1110101'
	retlw	b'1110001'
	retlw	b'1111111'
Charc:
	retlw	b'1110001'
	retlw	b'1110101'
	retlw	b'1110101'
	retlw	b'1111111'
Chard:
	retlw	b'1110001'
	retlw	b'1110101'
	retlw	b'1000001'
	retlw	b'1111111'
Chare:
	retlw	b'1110001'
	retlw	b'1110101'
	retlw	b'1111101'
	retlw	b'1111111'
Charf:
	retlw	b'1100001'
	retlw	b'1010111'
	retlw	b'1111111'
	retlw	b'1111111'
Charg:
	retlw	b'1100000'
	retlw	b'1101010'
	retlw	b'1100110'
	retlw	b'1111111'
Charh:
	retlw	b'1000001'
	retlw	b'1110111'
	retlw	b'1111001'
	retlw	b'1111111'
Chari:
	retlw	b'1111111'
	retlw	b'1010001'
	retlw	b'1111111'
	retlw	b'1111111'
Charj:
	retlw	b'1111110'
	retlw	b'1101000'
	retlw	b'1111111'
	retlw	b'1111111'
Chark:
	retlw	b'1000001'
	retlw	b'1110011'
	retlw	b'1101101'
	retlw	b'1111111'
Charl:
	retlw	b'1111111'
	retlw	b'1000001'
	retlw	b'1111111'
	retlw	b'1111111'
Charm:
	retlw	b'1110001'
	retlw	b'1111011'
	retlw	b'1111011'
	retlw	b'1110001'
Charn:
	retlw	b'1110001'
	retlw	b'1110111'
	retlw	b'1110001'
	retlw	b'1111111'
Charo:
	retlw	b'1110001'
	retlw	b'1110101'
	retlw	b'1110001'
	retlw	b'1111111'
Charp:
	retlw	b'1110000'
	retlw	b'1110101'
	retlw	b'1111011'
	retlw	b'1111111'
Charq:
	retlw	b'1111011'
	retlw	b'1110101'
	retlw	b'1110000'
	retlw	b'1111110'
Charr:
	retlw	b'1110001'
	retlw	b'1110111'
	retlw	b'1111111'
	retlw	b'1111111'
Chars:
	retlw	b'1110101'
	retlw	b'1101101'
	retlw	b'1101011'
	retlw	b'1111111'
Chart:
	retlw	b'1110111'
	retlw	b'1100001'
	retlw	b'1110111'
	retlw	b'1111111'
Charu:
	retlw	b'1110001'
	retlw	b'1111101'
	retlw	b'1110001'
	retlw	b'1111111'
Charv:
	retlw	b'1110011'
	retlw	b'1111101'
	retlw	b'1110011'
	retlw	b'1111111'
Charw:
	retlw	b'1100001'
	retlw	b'1111011'
	retlw	b'1111011'
	retlw	b'1100001'
Charx:
	retlw	b'1110101'
	retlw	b'1111011'
	retlw	b'1110101'
	retlw	b'1111111'
Chary:
	retlw	b'1110010'
	retlw	b'1111101'
	retlw	b'1110011'
	retlw	b'1111111'
Charz:
	retlw	b'1101001'
	retlw	b'1101101'
	retlw	b'1100101'
	retlw	b'1111111'


;;; =======================================================================
;;; =======================================================================

	END
